import platform
from os import path

import boto3



s3 = boto3.resource(
    service_name='s3',
    region_name='eu-west-2',
    aws_access_key_id='AKIAYHM23ZQQ4JOTA34C',
    aws_secret_access_key='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'
)

s3_bucket_path = '/vsis3/bci-physical-risk-model/'

elev_tif = 'WorldClim/Elevation/wc2.1_2.5m_elev.tif'

#future precipitation
CMIP6_ssp126_2021_2040 = 'WorldClim/FuturePrecipitation/wc2.1_2.5m_prec_CNRM-CM6-1_ssp126_2021-2040.tif'
CMIP6_ssp126_2041_2060 = 'WorldClim/FuturePrecipitation/wc2.1_2.5m_prec_CNRM-CM6-1_ssp126_2041-2060.tif'

CMIP6_ssp245_2021_2040 = 'WorldClim/FuturePrecipitation/wc2.1_2.5m_prec_CNRM-CM6-1_ssp245_2021-2040.tif'
CMIP6_ssp245_2041_2060 = 'WorldClim/FuturePrecipitation/wc2.1_2.5m_prec_CNRM-CM6-1_ssp245_2041-2060.tif'



from datetime import datetime

# for plotters
PLOT = False
FIGSIZE = (12, 9)

# for date
FIRST_HISTORICAL_FLOOD_DATE = datetime(1961, 1, 1)
MIN_DATE = datetime(1961, 1, 1)
MAX_DATE = datetime(2018, 12, 31)
FUTURE_MONTH = 0  # January
DAYS_ROLLBACK = 40  # total days are 41, 40+start_date

# for location:
BOUNDS_LONDON = [[-1, 51], [1, 52]]  # greater London
BOUNDS_UK = [[-8.4, 49.8], [1.9, 58.8]]  # UK incl. Northern Ireland

# for raster
RESOLUTION = 2.5
RADIUS = 1



import psycopg2 as pg
import pandas as pd

class DataQ():

    def __init__(self, query,
                 db="postgres", server="golden-source-2020.cyozpdhauzu4.us-east-2.rds.amazonaws.com",
                 user="postgres", pwd="Bcidatabase2020"):
        self.query      = query
        self.db         = db
        self.server     = server
        self.user       = user
        self.pwd        = pwd
        self.conn       = self.db_connect()
        self.data       = pd.read_sql(self.query, self.conn)
        self.describe   = self.data.describe()
        self.head       = self.data.head(10)
        self.db_disconnect()

    def db_connect(self):
        # add try/except
        conn = None
        try:
            conn = pg.connect(
                host=self.server,
                database=self.db,
                user=self.user,
                password=self.pwd)
        except Exception as err:
            print(err)
        return conn

    def db_disconnect(self):
        self.conn.close()
        pass


from timeit import default_timer as timer
import inspect
import re

class Formatter:
    # \033[  Escape code, this is always the same
    # 1 = Style, 1 for normal.
    # 32 = Text colour, 32 for bright green.
    # 40m = Background colour, 40 is for black.

    # class initialisation and stage flagging text
    @staticmethod
    def green(text):
        green='\033[0;32m'
        color_Off = '\033[0m'
        print(f'{green}{text}{color_Off}')

    @staticmethod
    def yellow(text):
        yellow ='\033[0;33m'
        color_Off = '\033[0m'
        print(f'{yellow}{text}{color_Off}')

    # Loading AWSdata format
    @staticmethod
    def cyan_background_with_white_text(text):
        cyan_white = '\033[0;30;46m'
        color_Off = '\033[0m'
        print(f'{cyan_white}{text}{color_Off}')

    # lambda expression
    @staticmethod
    def time_function_with_return(function):
        start = timer()
        result = function()
        end = timer()
        funcString = str(inspect.getsource(function))
        index = re.search('lambda: ', funcString).end() - 1
        funcString = funcString[index:len(funcString)-2]
        red ='\033[0;31m'
        color_off = '\033[0m'
        print(f'{red}The time spent on {funcString} is {end-start}{color_off}')
        return result

    # lambda expression
    @staticmethod
    def time_function_without_return(function):
        start = timer()
        function()
        end = timer()
        funcString = str(inspect.getsource(function))
        index = re.search('lambda: ', funcString).end() - 1
        funcString = funcString[index:len(funcString) - 2]
        red = '\033[0;31m'
        color_off = '\033[0m'
        print(f'{red}The time spent on{funcString} is {end-start}(s){color_off}')




import h2o
import pandas as pd
from os import path
import numpy as np
from typing import Tuple
from scipy.interpolate import interp1d
invalid = np.array(np.nan)

from physicalrisk.handler.build_prediction_data import Data_Handler
from physicalrisk.handler.printMessageHandler.formatePrint import Formatter
from physicalrisk.handler.database import DataQ

property_type = DataQ("select * from crrem.vw_epc_property_type").data
damage = DataQ("select * from physical.vw_flood_damage").data

class Building:
    def __init__(self, lon, lat, prop_type):
#         epc_prop = DataQ(f"""select * from public.epcsourcedata where "BUILDING_REFERENCE_NUMBER" = {building_id} """).data
#         epc_location = DataQ(f"""select * from public.epcaddresses where "BUILDING_REFERENCE_NUMBER" = {building_id} """).data        
#         self.lon_lat = [float(epc_location['PostcodeLongitude'].squeeze()), float(epc_location['PostcodeLatitude'].squeeze())]
#         self.prop_type = epc_prop['PROPERTY_TYPE'].squeeze()
        self.lon_lat = [lon, lat]
        self.prop_type = prop_type
    def predict_and_output(self, Longititude_Latitude: list, scenario):

        Construct_Features = Data_Handler()
        df_to_predict = Construct_Features.build_prediction_data(self.lon_lat, scenario)
        self.flood_depth = Construct_Features.flood_depth(self.lon_lat)
        
        h2o.init(log_level="ERRR")
        h2o_auto_model = h2o.import_mojo(path.dirname(__file__)+"/GBM_grid__1_AutoML_20210810_210013_model_4.zip")

        Formatter.green(f'Starting H2O_Auto_ML Prediction')

        h2odf_to_predict = h2o.H2OFrame(df_to_predict)
        result = h2o_auto_model.predict(h2odf_to_predict)
        result_df = h2o.as_list(result)
        probability = result_df['predict'].iloc[0]
        return probability

    def flood_depth(self):
        Construct_Features = Data_Handler()
        flood_depth = Construct_Features.flood_depth(self.lon_lat)
        return flood_depth
    
    def VAR(self,scenario):
        #calculate flood damage
        flood_damage_type = property_type[property_type['epc_prop_type']==self.prop_type]['prop_group_name'].squeeze()
        damage = DataQ("select * from physical.vw_flood_damage").data
        damage = damage[damage['prop_use_group_name']==flood_damage_type]
        x = damage['flood_depth']
        y = damage['flood_damage']
        f = interp1d(x, y, kind='quadratic')
        flood_damage = float(f(self.flood_depth()))

        probability = self.predict_and_output(self.lon_lat,scenario)
        VAR = flood_damage*probability
        return VAR